# This file is part of Androguard.
#
# Copyright (c) 2012 Geoffroy Gueguen <geoffroy.gueguen@gmail.com>
# All Rights Reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

import pydot
from dad.basic_blocks import (build_node_from_block, StatementBlock,
                              CondBlock, GenInvokeRetName)
from dad.util import log


class Graph():
    def __init__(self):
        self.entry = None
        self.exit = None
        self.nodes = set()
        self.rpo = []
        self.edges = {}
        self.reverse_edges = {}
        self.loc_to_ins = None
        self.loc_to_node = None

    def get_entry(self):
        return self.entry

    def get_exit(self):
        return self.exit

    def set_entry(self, node):
        self.entry = node

    def set_exit(self, node):
        self.exit = node

    def sucs(self, node):
        return self.edges.get(node, [])

    def preds(self, node):
        return self.reverse_edges.get(node, [])

    def get_rpo(self):
        return self.rpo

    def add_node(self, node):
        self.nodes.add(node)

    def add_edge(self, e1, e2):
        lsucs = self.edges.setdefault(e1, [])
        if e2 not in lsucs:
            lsucs.append(e2)
        lpreds = self.reverse_edges.setdefault(e2, [])
        if e1 not in lpreds:
            lpreds.append(e1)

    def remove_node(self, node):
        preds = self.reverse_edges.pop(node, [])
        for pred in preds:
            self.edges[pred].remove(node)

        succs = self.edges.get(node, [])
        for suc in succs:
            self.reverse_edges[suc].remove(node)

        self.nodes.remove(node)
        if node in self.rpo:
            self.rpo.remove(node)
        del node

    def number_ins(self):
        self.loc_to_ins = {}
        self.loc_to_node = {}
        num = 0
        for node in self.rpo:
            start_node = num
            num = node.number_ins(num)
            end_node = num - 1
            self.loc_to_ins.update(node.get_loc_with_ins())
            self.loc_to_node[(start_node, end_node)] = node

    def get_ins_from_loc(self, loc):
        return self.loc_to_ins.get(loc)

    def get_node_from_loc(self, loc):
        for (start, end), node in self.loc_to_node.iteritems():
            if start <= loc <= end:
                return node

    def remove_ins(self, loc):
        ins = self.get_ins_from_loc(loc)
        self.get_node_from_loc(loc).remove_ins(loc, ins)
        self.loc_to_ins.pop(loc)

    def split_if_nodes(self):
        '''
        Split IfNodes in two nodes, the first node is the header node, the
        second one is only composed of the jump condition.
        '''
        node_map = {}
        to_update = set()
        for node in self.nodes.copy():
            if node.is_cond():
                if len(node.get_ins()) > 1:
                    pre_ins = node.get_ins()[:-1]
                    last_ins = node.get_ins()[-1]
                    pre_node = StatementBlock('%s-pre' % node.name, pre_ins)
                    cond_node = CondBlock('%s-cond' % node.name, [last_ins])
                    node_map[node] = pre_node

                    pre_node.copy_from(node)
                    cond_node.copy_from(node)
                    pre_node.set_stmt()
                    cond_node.set_true(node.true)
                    cond_node.set_false(node.false)

                    lpreds = self.preds(node)
                    lsuccs = self.sucs(node)

                    for pred in lpreds:
                        pred_node = node_map.get(pred, pred)
                        if pred is node:
                            pred_node = cond_node
                        if pred.is_cond():  # and not (pred is node):
                            if pred.true is node:
                                pred_node.set_true(pre_node)
                            if pred.false is node:
                                pred_node.set_false(pre_node)
                        self.add_edge(pred_node, pre_node)
                    for suc in lsuccs:
                        self.add_edge(cond_node, node_map.get(suc, suc))

                    if node is self.entry:
                        self.set_entry(pre_node)

                    self.add_node(pre_node)
                    self.add_node(cond_node)
                    self.add_edge(pre_node, cond_node)
                    pre_node.update_attribute_with(node_map)
                    cond_node.update_attribute_with(node_map)
                    self.remove_node(node)
            else:
                to_update.add(node)
        for node in to_update:
            node.update_attribute_with(node_map)

    def simplify(self):
        '''
        Simplify the CFG by merging/deleting statement nodes when possible:
        If statement B follows statement A and if B has no other predecessor
        besides A, then we can merge A and B into a new statement node.
        We also remove nodes which do nothing except redirecting the control
        flow (nodes which only contains a goto).
        '''
        redo = True
        while redo:
            redo = False
            for node in self.nodes.copy():
                if node.is_stmt() and node in self.nodes:
                    suc = self.sucs(node)[0]
                    if len(node.get_ins()) == 0:
                        suc = self.edges.get(node)[0]
                        if node is suc:
                            continue
                        node_map = {node: suc}
                        for pred in self.preds(node):
                            pred.update_attribute_with(node_map)
                            self.add_edge(pred, suc)
                        redo = True
                        if node is self.entry:
                            self.entry = suc
                        self.remove_node(node)
                    elif (suc.is_stmt() and len(self.preds(suc)) == 1
                            and not ((node is suc) or (suc is self.entry))):
                        ins_to_merge = suc.get_ins()
                        node.add_ins(ins_to_merge)
                        new_suc = self.sucs(suc)[0]
                        if new_suc:
                            self.add_edge(node, new_suc)
                        redo = True
                        self.remove_node(suc)

    def compute_rpo(self):
        '''
        Number the nodes in reverse post order.
        An RPO traversal visit as many predecessors of a node as possible
        before visiting the node itself.
        If reverse is True, the RPO is done on the reverse graph.
        '''
        def traverse(node, visit, res):
            if node in visit:
                return
            visit.add(node)
            for suc in self.sucs(node):
                traverse(suc, visit, res)
            res.insert(0, node)
        visit = set()
        res = []
        traverse(self.entry, visit, res)
        for i, n in enumerate(res, 1):
            n.num = i
            self.rpo.append(n)

    def reset_rpo(self):
        self.rpo = []
        self.compute_rpo()

    def post_order(self, start=None, visited=None, res=None):
        '''
        Return the nodes of the graph in post-order i.e we visit all the
        children of a node before visiting the node itself.
        '''
        if visited is None:
            res = []
            visited = set()
            start = self.entry
        visited.add(start)
        for suc in self.sucs(start):
            if not suc in visited:
                self.post_order(suc, visited, res)
        res.append(start)
        return res

    def draw(self, name, dname, draw_branches=True):
        g = pydot.Dot()
        g.set_node_defaults(**{'color': 'lightgray', 'style': 'filled',
                    'shape': 'box', 'fontname': 'Courier', 'fontsize': '10'})
        for node in sorted(self.nodes, key=lambda x: x.num):
            if draw_branches and node.is_cond():
                edge_true = pydot.Edge(str(node), str(node.true))
                edge_false = pydot.Edge(str(node), str(node.false))
                edge_true.set_color('green')
                edge_false.set_color('red')
                g.add_edge(edge_true)
                g.add_edge(edge_false)
            else:
                succs = self.sucs(node)
                for suc in succs:
                    edge = pydot.Edge(str(node), str(suc))
                    edge.set_color('blue')
                    g.add_edge(edge)
        g.write_png('%s/%s.png' % (dname, name))

    def __len__(self):
        return len(self.nodes)

    def __repr__(self):
        return str(self.nodes)

    def __iter__(self):
        for node in self.nodes:
            yield node


def bfs(start):
    queue = [start]
    nodes = []
    nodes.append(start)
    while queue:
        node = queue.pop(0)
        for child in node.childs:
            if child[-1] not in nodes:
                nodes.append(child[-1])
                queue.append(child[-1])
    return nodes


def construct(basicblocks, vmap, exceptions):
    # Native methods... no blocks.
    if len(basicblocks) < 1:
        return log('Native Method.', 'debug')

    # Exceptions are not yet handled. An exception block has no parent, so
    # we can skip them by doing a BFS on the basic blocks.
    bfs_blocks = bfs(basicblocks[0])

    graph = Graph()
    gen_ret = GenInvokeRetName()

    # Construction of a mapping of basic blocks into Nodes
    block_to_node = {}
    for block in bfs_blocks:
        node = block_to_node.get(block)
        if node is None:
            node = build_node_from_block(block, block_to_node, vmap, gen_ret)
        for child in block.childs:
            child_block = child[-1]
            child_node = block_to_node.get(child_block)
            if child_node is None:
                child_node = build_node_from_block(child_block, block_to_node,
                                                               vmap, gen_ret)
            graph.add_edge(node, child_node)
            if node.is_switch():
                node.add_case(child_node)
            if node.is_cond():
                if_target = ((block.end / 2) - (block.last_length / 2)
                            + block.get_instructions()[-1].get_ref_off())
                child_addr = child_block.start / 2
                if if_target == child_addr:
                    node.set_true(child_node)
                else:
                    node.set_false(child_node)
        # Check that both branch of the if point to something
        # It may happen that both branch point to the same node, in this case
        # the false branch will be None. So we set it to the right node.
        # TODO: In this situation, we should transform the condition node into
        # a statement node
        if node.is_cond() and node.false is None:
            node.set_false(node.true)

        graph.add_node(node)

    graph.set_entry(block_to_node[basicblocks[0]])

    graph.compute_rpo()
    graph.number_ins()

    # Create a list of Node which are 'return' node
    # There should be one and only one node of this type
    # If this is not the case, try to continue anyway by setting the exit node
    # to the one which has the greatest RPO number (not necessarily the case)
    lexit_nodes = [node for node in graph if node.is_return()]

    if len(lexit_nodes) > 1:
        # Not sure that this case is possible...
        log('Multiple exit nodes found !', 'error')
        graph.set_exit(graph.get_rpo()[-1])
    elif len(lexit_nodes) < 1:
        # A method can have no return if it has throw statement(s) or if its
        # body is a while(1) whitout break/return.
        log('No exit node found !', 'debug')
    else:
        graph.set_exit(lexit_nodes[0])

    return graph



#!/usr/bin/env python

# This file is part of Androguard.
#
# Copyright (C) 2012, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

from xml.sax.saxutils import escape
import sys, os
from optparse import OptionParser

from androguard.core.androgen import Androguard
from androguard.core.analysis import analysis

option_0 = { 'name' : ('-i', '--input'), 'help' : 'filename input', 'nargs' : 1 }
option_1 = { 'name' : ('-o', '--output'), 'help' : 'filename output of the xgmml', 'nargs' : 1 }
option_2 = { 'name' : ('-f', '--functions'), 'help' : 'include function calls', 'action' : 'count' }
option_3 = { 'name' : ('-e', '--externals'), 'help' : 'include extern function calls', 'action' : 'count' }
option_4 = { 'name' : ('-v', '--version'), 'help' : 'version of the API', 'action' : 'count' }

options = [option_0, option_1, option_2, option_3, option_4]

METHODS_ID = {}
EXTERNAL_METHODS_ID = {}
NODES_ID = {}
EDGES_ID = {}

NODE_GRAPHIC = {
   "classic" : {
                 "h" : 20.0,
                 "w" : 20.0,
                 "type" : "ELLIPSE",
                 "width" : 1,
                 "fill" : "#e1e1e1",
                 "outline" : "#000000",
               },

   "extern" : {
                 "h" : 20.0,
                 "w" : 20.0,
                 "type" : "ELLIPSE",
                 "width" : 1,
                 "fill" : "#ff8c00",
                 "outline" : "#000000",
               }
}

EDGE_GRAPHIC = {
   "cfg" : {
               "width" : 2,
               "fill" : "#0000e1",
   },

   "fcg" : {
               "width" : 3,
               "fill" : "#9acd32",
   },

   "efcg" : {
               "width" : 3,
               "fill" : "#808000",
   }
}

def get_node_name(method, bb) :
    return "%s-%s-%s" % ( method.get_class_name(), escape(bb.name), escape(method.get_descriptor()) )

def export_xgmml_cfg(g, fd) :
    method = g.get_method()

    name = method.get_name()
    class_name = method.get_class_name()
    descriptor = method.get_descriptor()

    if method.get_code() != None :
        size_ins = method.get_code().get_length()

    for i in g.basic_blocks.get() :
        fd.write("<node id=\"%d\" label=\"%s\">\n" % (len(NODES_ID), get_node_name(method, i)))

        fd.write("<att type=\"string\" name=\"classname\" value=\"%s\"/>\n" % (escape(class_name)))
        fd.write("<att type=\"string\" name=\"name\" value=\"%s\"/>\n" % (escape(name)))
        fd.write("<att type=\"string\" name=\"descriptor\" value=\"%s\"/>\n" % (escape(descriptor)))

        fd.write("<att type=\"integer\" name=\"offset\" value=\"%d\"/>\n" % (i.start))

        cl = NODE_GRAPHIC["classic"]
        width = cl["width"]
        fill = cl["fill"]

        # No child ...
        if i.childs == [] :
            fill = "#87ceeb"

        if i.start == 0 :
            fd.write("<att type=\"string\" name=\"node.label\" value=\"%s\\n%s\"/>\n" % (escape(name), i.get_ins()[-1].get_name()))
            width = 3
            fill = "#ff0000"

            METHODS_ID[ class_name + name + descriptor ] = len(NODES_ID)
        else :
            fd.write("<att type=\"string\" name=\"node.label\" value=\"0x%x\\n%s\"/>\n" % (i.start, i.get_ins()[-1].get_name()))

        size = 0
        for tmp_ins in i.get_ins() :
            size += (tmp_ins.get_length() / 2)


        h = ((size / float(size_ins)) * 20) + cl["h"]

        fd.write("<graphics type=\"%s\" h=\"%.1f\" w=\"%.1f\" width=\"%d\" fill=\"%s\" outline=\"%s\">\n" % ( cl["type"], h, h, width, fill, cl["outline"]))
        fd.write("</graphics>\n")

        fd.write("</node>\n")

        NODES_ID[ class_name + i.name + descriptor ] = len(NODES_ID)

    for i in g.basic_blocks.get() :
        for j in i.childs :
            if j[-1] != None :
                label = "%s (cfg) %s" % (get_node_name(method, i), get_node_name(method, j[-1]))
                id = len(NODES_ID) + len(EDGES_ID)
                fd.write( "<edge id=\"%d\" label=\"%s\" source=\"%d\" target=\"%d\">\n" % (id, label, NODES_ID[ class_name + i.name + descriptor ], NODES_ID[ class_name + j[-1].name + descriptor ]) )

                cl = EDGE_GRAPHIC["cfg"]
                fd.write("<graphics width=\"%d\" fill=\"%s\">\n" % (cl["width"], cl["fill"]) )
                fd.write("</graphics>\n")

                fd.write("</edge>\n")

                EDGES_ID[ label ] = id

def export_xgmml_fcg(a, x, fd) :
    classes = a.get_classes_names()

    # Methods flow graph
    for m, _ in x.get_tainted_packages().get_packages() :
        paths = m.get_methods()
        for j in paths :
            if j.get_method().get_class_name() in classes and m.get_info() in classes :
                if j.get_access_flag() == analysis.TAINTED_PACKAGE_CALL :
                    t =  m.get_info() + j.get_name() + j.get_descriptor()
                    if t not in METHODS_ID :
                        continue

                    bb1 = x.get_method( j.get_method() ).basic_blocks.get_basic_block( j.get_idx() )

                    node1 = get_node_name(j.get_method(), bb1) + "@0x%x" % j.get_idx()
                    node2 = "%s-%s-%s" % (m.get_info(), escape(j.get_name()), escape(j.get_descriptor()))

                    label = "%s (fcg) %s" % (node1, node2)

                    if label in EDGES_ID :
                        continue

                    id = len(NODES_ID) + len(EDGES_ID)

                    fd.write( "<edge id=\"%d\" label=\"%s\" source=\"%d\" target=\"%d\">\n" % (id,
                                                                                               label,
                                                                                               NODES_ID[ j.get_method().get_class_name() + bb1.name + j.get_method().get_descriptor() ],
                                                                                               METHODS_ID[ m.get_info() + j.get_name() + j.get_descriptor() ]) )

                    cl = EDGE_GRAPHIC["fcg"]
                    fd.write("<graphics width=\"%d\" fill=\"%s\">\n" % (cl["width"], cl["fill"]) )
                    fd.write("</graphics>\n")

                    fd.write("</edge>\n")

                    EDGES_ID[ label ] = id

def export_xgmml_efcg(a, x, fd) :
    classes = a.get_classes_names()

    # Methods flow graph
    for m, _ in x.get_tainted_packages().get_packages() :
        paths = m.get_methods()
        for j in paths :
            if j.get_method().get_class_name() in classes and m.get_info() not in classes :
                if j.get_access_flag() == analysis.TAINTED_PACKAGE_CALL :
                    t =  m.get_info() + j.get_name() + j.get_descriptor()
                    if t not in EXTERNAL_METHODS_ID :
                        fd.write("<node id=\"%d\" label=\"%s\">\n" % (len(NODES_ID), escape(t)))

                        fd.write("<att type=\"string\" name=\"classname\" value=\"%s\"/>\n" % (escape(m.get_info())))
                        fd.write("<att type=\"string\" name=\"name\" value=\"%s\"/>\n" % (escape(j.get_name())))
                        fd.write("<att type=\"string\" name=\"descriptor\" value=\"%s\"/>\n" % (escape(j.get_descriptor())))

                        cl = NODE_GRAPHIC["extern"]

                        fd.write("<att type=\"string\" name=\"node.label\" value=\"%s\\n%s\\n%s\"/>\n" % (escape(m.get_info()), escape(j.get_name()), escape(j.get_descriptor())))

                        fd.write("<graphics type=\"%s\" h=\"%.1f\" w=\"%.1f\" width=\"%d\" fill=\"%s\" outline=\"%s\">\n" % ( cl["type"], cl["h"], cl["h"], cl["width"], cl["fill"], cl["outline"]))
                        fd.write("</graphics>\n")

                        fd.write("</node>\n")

                        NODES_ID[ t ] = len(NODES_ID)
                        EXTERNAL_METHODS_ID[ t ] = NODES_ID[ t ]

                    bb1 = x.get_method( j.get_method() ).basic_blocks.get_basic_block( j.get_idx() )

                    node1 = get_node_name(j.get_method(), bb1) + "@0x%x" % j.get_idx()
                    node2 = "%s-%s-%s" % (m.get_info(), escape(j.get_name()), escape(j.get_descriptor()))

                    label = "%s (efcg) %s" % (node1, node2)

                    if label in EDGES_ID :
                        continue

                    id = len(NODES_ID) + len(EDGES_ID)

                    fd.write( "<edge id=\"%d\" label=\"%s\" source=\"%d\" target=\"%d\">\n" % (id,
                                                                                               label,
                                                                                               NODES_ID[ j.get_method().get_class_name() + bb1.name + j.get_method().get_descriptor() ],
                                                                                               EXTERNAL_METHODS_ID[ m.get_info() + j.get_name() + j.get_descriptor() ]) )

                    cl = EDGE_GRAPHIC["efcg"]
                    fd.write("<graphics width=\"%d\" fill=\"%s\">\n" % (cl["width"], cl["fill"]) )
                    fd.write("</graphics>\n")

                    fd.write("</edge>\n")

                    EDGES_ID[ label ] = id

def export_apps_to_xgmml( input, output, fcg, efcg ) :
    a = Androguard( [ input ] )

    fd = open(output, "w")
    fd.write("<?xml version='1.0'?>\n")
    fd.write("<graph label=\"Androguard XGMML %s\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:ns1=\"http://www.w3.org/1999/xlink\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns=\"http://www.cs.rpi.edu/XGMML\" directed=\"1\">\n" % (os.path.basename(input)))

    for vm in a.get_vms() :
        x = analysis.VMAnalysis( vm )
        # CFG
        for method in vm.get_methods() :
            g = x.get_method( method )
            export_xgmml_cfg(g, fd)

        if fcg :
            export_xgmml_fcg(vm, x, fd)

        if efcg :
            export_xgmml_efcg(vm, x, fd)

    fd.write("</graph>")
    fd.close()

def main(options, arguments) :
    if options.input != None and options.output != None :
        export_apps_to_xgmml( options.input, options.output, options.functions, options.externals )

if __name__ == "__main__" :
    parser = OptionParser()
    for option in options :
        param = option['name']
        del option['name']
        parser.add_option(*param, **option)


    options, arguments = parser.parse_args()
    sys.argv[:] = arguments
    main(options, arguments)



#!/usr/bin/env python

# This file is part of Androguard.
#
# Copyright (C) 2012, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

import sys

from optparse import OptionParser

from androguard.core.bytecodes import apk, dvm
from androguard.core.analysis import analysis
from androguard.core import androconf

sys.path.append("./elsim")
from elsim import elsim
from elsim.elsim_dalvik import ProxyDalvik, FILTERS_DALVIK_SIM, ProxyDalvikMethod, FILTERS_DALVIK_BB
from elsim.elsim_dalvik import ProxyDalvikBasicBlock, FILTERS_DALVIK_DIFF_BB
from elsim.elsim_dalvik import DiffDalvikMethod


option_0 = { 'name' : ('-i', '--input'), 'help' : 'file : use these filenames', 'nargs' : 2 }
option_1 = { 'name' : ('-t', '--threshold'), 'help' : 'define the threshold', 'nargs' : 1 }
option_2 = { 'name' : ('-c', '--compressor'), 'help' : 'define the compressor', 'nargs' : 1 }
option_3 = { 'name' : ('-d', '--display'), 'help' : 'display the file in human readable format', 'action' : 'count' }
#option_4 = { 'name' : ('-e', '--exclude'), 'help' : 'exclude specific blocks (0 : orig, 1 : diff, 2 : new)', 'nargs' : 1 }
option_5 = { 'name' : ('-e', '--exclude'), 'help' : 'exclude specific class name (python regexp)', 'nargs' : 1 }
option_6 = { 'name' : ('-s', '--size'), 'help' : 'exclude specific method below the specific size', 'nargs' : 1 }
option_7 = { 'name' : ('-v', '--version'), 'help' : 'version of the API', 'action' : 'count' }

options = [option_0, option_1, option_2, option_3, option_5, option_6, option_7]

def main(options, arguments) :
    details = False
    if options.display != None :
        details = True
    
    if options.input != None :
        ret_type = androconf.is_android( options.input[0] )
        if ret_type == "APK" :
            a = apk.APK( options.input[0] )
            d1 = dvm.DalvikVMFormat( a.get_dex() )
        elif ret_type == "DEX" :
            d1 = dvm.DalvikVMFormat( open(options.input[0], "rb").read() )
        
        dx1 = analysis.VMAnalysis( d1 )
       
        ret_type = androconf.is_android( options.input[1] )
        if ret_type == "APK" :
            a = apk.APK( options.input[1] )
            d2 = dvm.DalvikVMFormat( a.get_dex() )
        elif ret_type == "DEX" :
            d2 = dvm.DalvikVMFormat( open(options.input[1], "rb").read() )
        
        dx2 = analysis.VMAnalysis( d2 )

        print d1, dx1, d2, dx2
        sys.stdout.flush()
        
        threshold = None
        if options.threshold != None :
            threshold = float(options.threshold)

        FS = FILTERS_DALVIK_SIM
        FS[elsim.FILTER_SKIPPED_METH].set_regexp( options.exclude )
        FS[elsim.FILTER_SKIPPED_METH].set_size( options.size )
        el = elsim.Elsim( ProxyDalvik(d1, dx1), ProxyDalvik(d2, dx2), FS, threshold, options.compressor )
        el.show()

        e1 = elsim.split_elements( el, el.get_similar_elements() )
        for i in e1 :
            j = e1[ i ]
            elb = elsim.Elsim( ProxyDalvikMethod(i), ProxyDalvikMethod(j), FILTERS_DALVIK_BB, threshold, options.compressor )
            #elb.show()

            eld = elsim.Eldiff( ProxyDalvikBasicBlock(elb), FILTERS_DALVIK_DIFF_BB )
            #eld.show()

            ddm = DiffDalvikMethod( i, j, elb, eld )
            ddm.show()

        print "NEW METHODS"
        enew = el.get_new_elements()
        for i in enew :
            el.show_element( i, False )

        print "DELETED METHODS"
        edel = el.get_deleted_elements()
        for i in edel :
            el.show_element( i )

    elif options.version != None :
        print "Androdiff version %s" % androconf.ANDROGUARD_VERSION

if __name__ == "__main__" :
    parser = OptionParser()
    for option in options :
        param = option['name']
        del option['name']
        parser.add_option(*param, **option)

    options, arguments = parser.parse_args()
    sys.argv[:] = arguments
    main(options, arguments)



# This file is part of Androguard.
#
# Copyright (c) 2012 Geoffroy Gueguen <geoffroy.gueguen@gmail.com>
# All Rights Reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

from dad.util import log
from dad.instruction import Constant, BinaryCompExpression


class Visitor(object):
    def __init__(self, graph):
        self.graph = graph
        self.visited_nodes = set()
        self.loop_follow = [None]
        self.latch_node = [None]
        self.if_follow = [None]
        self.switch_follow = [None]
        self.next_case = None

    def visit_ins(self, ins):
        ins.visit(self)

    def visit_node(self, node):
        if node in (self.if_follow[-1], self.switch_follow[-1],
                    self.loop_follow[-1], self.latch_node[-1]):
            return
        if node in self.visited_nodes:
            return
        self.visited_nodes.add(node)
        node.visit(self)

    def visit_loop_node(self, loop):
        follow = loop.get_loop_follow()
        if follow is None and not loop.looptype.endless():
            log('Loop has no follow !', 'error')
        if loop.looptype.pretest():
            if loop.true is follow:
                loop.neg()
                loop.true, loop.false = loop.false, loop.true
            loop.visit_cond(self)
        elif loop.looptype.posttest():
            self.latch_node.append(loop.latch)
        elif loop.looptype.endless():
            pass
        self.loop_follow.append(follow)
        if loop.looptype.pretest():
            self.visit_node(loop.true)
        else:
            self.visit_node(loop.cond)
        self.loop_follow.pop()
        if loop.looptype.pretest():
            pass
        elif loop.looptype.posttest():
            self.latch_node.pop()
            loop.latch.visit_cond(self)
        else:
            self.visit_node(loop.latch)
        if follow is not None:
            self.visit_node(follow)

    def visit_cond_node(self, cond):
        follow = cond.get_if_follow()
        if cond.false is self.loop_follow[-1]:
            cond.neg()
            cond.true, cond.false = cond.false, cond.true
            cond.visit_cond(self)
            self.visit_node(cond.false)
        elif follow is not None:
            is_else = not (follow in (cond.true, cond.false))
            if (cond.true in (follow, self.next_case)
                                                or cond.num > cond.true.num):
                cond.neg()
                cond.true, cond.false = cond.false, cond.true
            self.if_follow.append(follow)
            if not cond.true in self.visited_nodes:
                cond.visit_cond(self)
                self.visit_node(cond.true)
            if is_else and not cond.false in self.visited_nodes:
                self.visit_node(cond.false)
            self.if_follow.pop()
            self.visit_node(follow)
        else:
            cond.visit_cond(self)
            self.visit_node(cond.true)
            self.visit_node(cond.false)

    def visit_short_circuit_condition(self, nnot, aand, cond1, cond2):
        if nnot:
            cond1.neg()
        cond1.visit_cond(self)
        cond2.visit_cond(self)

    def visit_switch_node(self, switch):
        lins = switch.get_ins()
        for ins in lins[:-1]:
            self.visit_ins(ins)
        switch_ins = switch.get_ins()[-1]
        self.visit_ins(switch_ins)
        follow = switch.switch_follow
        cases = switch.cases
        self.switch_follow.append(follow)
        default = switch.default
        for i, node in enumerate(cases):
            if node in self.visited_nodes:
                continue
            for case in switch.node_to_case[node]:
                pass
            if i + 1 < len(cases):
                self.next_case = cases[i + 1]
            else:
                self.next_case = None
            if node is default:
                default = None
            self.visit_node(node)
        if default not in (None, follow):
            self.visit_node(default)
        self.switch_follow.pop()
        self.visit_node(follow)

    def visit_statement_node(self, stmt):
        sucs = self.graph.sucs(stmt)
        for ins in stmt.get_ins():
            self.visit_ins(ins)
        if len(sucs) == 0:
            return
        follow = sucs[0]
        self.visit_node(follow)

    def visit_return_node(self, ret):
        for ins in ret.get_ins():
            self.visit_ins(ins)

    def visit_throw_node(self, throw):
        for ins in throw.get_ins():
            self.visit_ins(ins)

    def visit_constant(self, cst):
        pass

    def visit_base_class(self, cls):
        pass

    def visit_variable(self, var):
        pass

    def visit_param(self, param):
        pass

    def visit_this(self):
        pass

    def visit_assign(self, lhs, rhs):
        if lhs is None:
            rhs.visit(self)
            return
        lhs.visit(self)
        rhs.visit(self)

    def visit_move_result(self, lhs, rhs):
        lhs.visit(self)
        rhs.visit(self)

    def visit_move(self, lhs, rhs):
        if lhs is rhs:
            return
        lhs.visit(self)
        rhs.visit(self)

    def visit_astore(self, array, index, rhs):
        array.visit(self)
        if isinstance(index, Constant):
            index.visit(self, 'I')
        else:
            index.visit(self)
        rhs.visit(self)

    def visit_put_static(self, cls, name, rhs):
        rhs.visit(self)

    def visit_put_instance(self, lhs, name, rhs):
        lhs.visit(self)
        rhs.visit(self)

    def visit_new(self, atype):
        pass

    def visit_invoke(self, name, base, args):
        base.visit(self)
        for arg in args:
            arg.visit(self)

    def visit_return_void(self):
        pass

    def visit_return(self, arg):
        arg.visit(self)

    def visit_nop(self):
        pass

    def visit_switch(self, arg):
        arg.visit(self)

    def visit_check_cast(self, arg, atype):
        arg.visit(self)

    def visit_aload(self, array, index):
        array.visit(self)
        index.visit(self)

    def visit_alength(self, array):
        array.visit(self)

    def visit_new_array(self, atype, size):
        size.visit(self)

    def visit_filled_new_array(self, atype, size, args):
        atype.visit(self)
        size.visit(self)
        for arg in args:
            arg.visit(self)

    def visit_fill_array(self, array, value):
        array.visit(self)

    def visit_monitor_enter(self, ref):
        ref.visit(self)

    def visit_monitor_exit(self, ref):
        pass

    def visit_throw(self, ref):
        ref.visit(self)

    def visit_binary_expression(self, op, arg1, arg2):
        arg1.visit(self)
        arg2.visit(self)

    def visit_unary_expression(self, op, arg):
        arg.visit(self)

    def visit_cast(self, op, arg):
        arg.visit(self)

    def visit_cond_expression(self, op, arg1, arg2):
        arg1.visit(self)
        arg2.visit(self)

    def visit_condz_expression(self, op, arg):
        if isinstance(arg, BinaryCompExpression):
            arg.op = op
            arg.visit(self)
        else:
            arg.visit(self)

    def visit_get_instance(self, arg, name):
        arg.visit(self)

    def visit_get_static(self, cls, name):
        pass



#!/usr/bin/env python

# This file is part of Androguard.
#
# Copyright (C) 2012, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

import sys, os

from optparse import OptionParser

from androguard.core import androconf
from androguard.core.bytecodes import apk
from androguard.core.bytecodes import dvm
from androguard.core.analysis import analysis 

option_0 = { 'name' : ('-i', '--input'), 'help' : 'file : use this filename (APK)', 'nargs' : 1 }
option_1 = { 'name' : ('-d', '--directory'), 'help' : 'directory : use this directory', 'nargs' : 1 }
option_2 = { 'name' : ('-t', '--tag'), 'help' : 'display tags', 'action' : 'count' }
option_3 = { 'name' : ('-v', '--version'), 'help' : 'version', 'action' : 'count' }

options = [option_0, option_1, option_2, option_3]

def display_dvm_info(apk) :
    vm = dvm.DalvikVMFormat( apk.get_dex() )
    vmx = analysis.uVMAnalysis( vm )

    print "Native code:", analysis.is_native_code(vmx)
    print "Dynamic code:", analysis.is_dyn_code(vmx)
    print "Reflection code:", analysis.is_reflection_code(vmx)

    for i in vmx.get_methods() :
      i.create_tags()
      if not i.tags.empty() :
        print i.method.get_class_name(), i.method.get_name(), i.tags

def main(options, arguments) :
    if options.input != None :
        ret_type = androconf.is_android( options.input ) 

        print os.path.basename(options.input), ":"
        if ret_type == "APK" :
            try :
                a = apk.APK( options.input )
                if a.is_valid_APK() :
                    a.show()
                    display_dvm_info( a )
                else :
                    print "INVALID"
            except Exception, e :
                print "ERROR", e
                
    elif options.directory != None :
        for root, dirs, files in os.walk( options.directory, followlinks=True ) :
            if files != [] :
                for f in files :
                    real_filename = root
                    if real_filename[-1] != "/" :
                        real_filename += "/"
                    real_filename += f

                    ret_type = androconf.is_android( real_filename )
                    if ret_type == "APK"  :
                        print os.path.basename( real_filename ), ":"
                        try :
                            a = apk.APK( real_filename )
                            if a.is_valid_APK() :
                                a.show()
                                display_dvm_info( a )
                            else :
                                print "INVALID APK"
                                raise("ooos")
                        except Exception, e :
                            print "ERROR", e
                            raise("ooos")

    elif options.version != None :
        print "Androapkinfo version %s" % androconf.ANDROGUARD_VERSION

if __name__ == "__main__" :
    parser = OptionParser()
    for option in options :
        param = option['name']
        del option['name']
        parser.add_option(*param, **option)

    options, arguments = parser.parse_args()
    sys.argv[:] = arguments
    main(options, arguments)



# This file is part of Androguard.
#
# Copyright (C) 2012, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.


from idaapi import *
from idautils import *
from idc import *

from SimpleXMLRPCServer import SimpleXMLRPCServer
import cPickle

def is_connected() :
    return True 

def wrapper_get_raw(oops) :
    F = {}
    for function_ea in Functions() :

        F[ function_ea ] = []

        f_start = function_ea
        f_end = GetFunctionAttr(function_ea, FUNCATTR_END)
        
        edges = set()
        boundaries = set((f_start,))

        F[ function_ea ].append( GetFunctionName(function_ea) )

        for head in Heads(f_start, f_end) :
            if isCode( GetFlags( head ) ) :
                F[ function_ea ].append( (head, GetMnem(head), GetOpnd(head, 0), GetOpnd(head, 1), GetOpnd(head, 2)) )

                refs = CodeRefsFrom(head, 0)
                refs = set(filter(lambda x: x>=f_start and x<=f_end, refs))
                
                if refs :
                    next_head = NextHead(head, f_end)
                    if isFlow(GetFlags(next_head)):
                        refs.add(next_head)
                                                      
                    # Update the boundaries found so far.
                    boundaries.update(refs)
                                                                                                  
                    # For each of the references found, and edge is
                    # created.
                    for r in refs:
                    # If the flow could also come from the address
                    # previous to the destination of the branching
                    # an edge is created.
                        if isFlow(GetFlags(r)):
                            edges.add((PrevHead(r, f_start), r))
                        edges.add((head, r))
        
        #print edges, boundaries
        # Let's build the list of (startEA, startEA) couples
        # for each basic block
        sorted_boundaries = sorted(boundaries, reverse = True)
        end_addr = PrevHead(f_end, f_start)
        bb_addr = []
        for begin_addr in sorted_boundaries:
            bb_addr.append((begin_addr, end_addr))
            # search the next end_addr which could be
            # farther than just the previous head
            # if data are interlaced in the code
            # WARNING: it assumes it won't epicly fail ;)
            end_addr = PrevHead(begin_addr, f_start)
            while not isCode(GetFlags(end_addr)):
                end_addr = PrevHead(end_addr, f_start)
        # And finally return the result
        bb_addr.reverse()
        F[ function_ea ].append( (bb_addr, sorted(edges)) )

    return cPickle.dumps( F )

def wrapper_Heads(oops) :
    start, end = cPickle.loads(oops)
    return cPickle.dumps( [ x for x in Heads( start, end ) ] )

def wrapper_Functions(oops) :
    return cPickle.dumps( [ x for x in Functions() ] )

def wrapper_get_function(oops) :
    name = cPickle.loads(oops)
    for function_ea in Functions() :
        if GetFunctionName(function_ea) == name :
            return cPickle.dumps( function_ea )
    return cPickle.dumps( -1 )

def wrapper_quit(oops) :
    qexit(0)

class IDAWrapper :
    def _dispatch(self, x, params) :
        #fd = open("toto.txt", "w")
        #fd.write( x + "\n" )
        #fd.write( str(type(params[0])) + "\n" )
        #fd.close()
        
        params = cPickle.loads( *params )
        if isinstance(params, tuple) == False :
            params = (params,)

        import types
        import idautils
        import idc

        #[getattr(idautils, a, None) for a in dir(idautils) if isinstance(getattr(idautils, a, None) , types.FunctionType)]
        for a in dir(idautils) :
            #fd.write( "\t" + a + "\n" )
            if a == x :
                z = getattr(idautils, a, None)
                ret = z( *params )
                if type(ret).__name__=='generator' :
                    return cPickle.dumps( [ i for i in ret ] )
                return cPickle.dumps( ret )

        for a in dir(idc) :
            #fd.write( "\t" + a + "\n" )
            if a == x :
                z = getattr(idc, a, None)
                ret = z( *params )
                if type(ret).__name__=='generator' :
                    return cPickle.dumps( [ i for i in ret ] )
                return cPickle.dumps( ret )

        return cPickle.dumps( [] )

def main() :
    autoWait()
    ea = ScreenEA()

    server = SimpleXMLRPCServer(("localhost", 9000))
    server.register_function(is_connected, "is_connected")
    
    server.register_function(wrapper_get_raw, "get_raw")
    server.register_function(wrapper_get_function, "get_function")
    server.register_function(wrapper_Heads, "Heads")
    server.register_function(wrapper_Functions, "Functions")
    
    server.register_instance(IDAWrapper())
    
    server.register_function(wrapper_quit, "quit")
    server.serve_forever()

    qexit(0)

main()



# This file is part of Androguard.
#
# Copyright (c) 2012 Geoffroy Gueguen <geoffroy.gueguen@gmail.com>
# All Rights Reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

from dad.util import build_path, common_dom


def dominance_frontier(graph, immdoms):
    '''
    Create the dominance frontier of each nodes of the graph.
    The dominance frontier of a node n is the set of all nodes m such that
    n dominates an immediate predecessor of m but does not strictly dominate m.
    '''
    DF = {}
    for node in graph:
        DF[node] = set()
    for node in graph:
        # Nodes in a DF set must be join points in the graph
        preds = graph.preds(node)
        if len(preds) > 1:
            # We found a join point. Now for each of its predecessor we walk up
            # the dominator tree to find a node that dominates it.
            # The join point belong to the DF of all the nodes which are on the
            # dominator tree walk.
            for pred in preds:
                runner = pred
                while runner != immdoms[node]:
                    DF[runner].add(node)
                    runner = immdoms[runner]
    return DF


class BasicReachDef(object):
    def __init__(self, graph, params):
        self.g = graph
        self.A = {}
        self.R = {}
        self.DB = {}
        self.defs = {}
        self.def_to_loc = {}
        # Deal with special entry node
        entry = graph.get_entry()
        self.defs[entry] = {}
        self.A[entry] = set([-1])
        for param in params:
            self.defs[entry][param] = set([-1])
            self.def_to_loc[param] = set([-1])
        # Deal with the other nodes
        for node in graph.get_rpo():
            self.A[node] = set()
            self.R[node] = set()
            self.DB[node] = set()
            self.defs.setdefault(node, dict())
            for i, ins in node.get_loc_with_ins():
                kill = ins.get_lhs()
                if kill is not None:
                    self.defs[node].setdefault(kill, set()).add(i)
                    self.def_to_loc.setdefault(kill, set()).add(i)
            for defs, values in self.defs[node].items():
                self.DB[node].add(max(values))

    def run(self):
        nodes = self.g.get_rpo()[:]
        while nodes:
            node = nodes.pop(0)
            predA = [self.A[p] for p in self.g.preds(node)]
            newR = reduce(set.union, predA, set())
            if predA and newR != self.R[node]:
                self.R[node] = newR
                for suc in self.g.sucs(node):
                    if suc not in nodes:
                        nodes.append(suc)

            killed_locs = set()
            for reg in self.defs[node]:
                for loc in self.def_to_loc[reg]:
                    killed_locs.add(loc)

            A = set()
            for loc in self.R[node]:
                if loc not in killed_locs:
                    A.add(loc)
            newA = A.union(self.DB[node])
            if newA != self.A[node]:
                self.A[node] = newA
                for suc in self.g.sucs(node):
                    if suc not in nodes:
                        nodes.append(suc)


def update_chain(graph, loc, du, ud):
    '''
    Updates the DU chain of the instruction located at loc such that there is
    no more reference to it so that we can remove it.
    When an instruction is found to be dead (i.e it has no side effect, and the
    register defined is not used) we have to update the DU chain of all the
    variables that may me used by the dead instruction.
    '''
    ins = graph.get_ins_from_loc(loc)
    for var in ins.get_used_vars():
        # We get the definition points of the current variable
        for def_loc in set(ud.get((var, loc), ())):
            # We remove the use of the variable at loc from the DU chain of
            # the variable definition located at def_loc
            du.get((var, def_loc)).remove(loc)
            # If the DU chain of the defined variable is now empty, this means
            # that we may have created a new dead instruction, so we check that
            # the instruction has no side effect and we update the DU chain of
            # the new dead instruction, and we delete it.
            # We also make sure that def_loc is not -1. This is the case when
            # the current variable is a method parameter.
            if  def_loc >= 0 and len(du.get((var, def_loc))) == 0:
                def_ins = graph.get_ins_from_loc(def_loc)
                if def_ins.is_call():
                    def_ins.remove_defined_var()
                elif def_ins.has_side_effect():
                    continue
                else:
                    update_chain(graph, def_loc, du, ud)
                    graph.remove_ins(def_loc)


def dead_code_elimination(graph, du, ud):
    '''
    Run a dead code elimination pass.
    Instructions are checked to be dead. If it is the case, we remove them and
    we update the DU & UD chains of its variables to check for further dead
    instructions.
    '''
    for node in graph.get_rpo():
        for i, ins in node.get_loc_with_ins()[:]:
            reg = ins.get_lhs()
            if reg is not None:
                # If the definition is not used, we check that the instruction
                # has no side effect. If there is one and this is a call, we
                # remove only the unused defined variable. else, this is
                # something like an array access, so we do nothing.
                # Otherwise (no side effect) we can remove the instruction from
                # the node.
                if du.get((reg, i), None) is  None:
                    if ins.is_call():
                        ins.remove_defined_var()
                    elif ins.has_side_effect():
                        continue
                    else:
                        # We can delete the instruction. First update the DU
                        # chain of the variables used by the instruction to
                        # `let them know` that they are not used anymore by the
                        # deleted instruction.
                        # Then remove the instruction.
                        update_chain(graph, i, du, ud)
                        graph.remove_ins(i)


def clear_path_node(graph, reg, loc1, loc2):
    for loc in xrange(loc1, loc2):
        ins = graph.get_ins_from_loc(loc)
        if ins is None:
            continue
        if ins.get_lhs() == reg or ins.has_side_effect():
            return False
    return True


def clear_path(graph, reg, loc1, loc2):
    '''
    Check that the path from loc1 to loc2 is clear.
    We have to check that there is no side effect between the two location
    points. We also have to check that the variable `reg` is not redefined
    along one of the possible pathes from loc1 to loc2.
    '''
    node1 = graph.get_node_from_loc(loc1)
    node2 = graph.get_node_from_loc(loc2)
    # If both instructions are in the same node, we only have to check that the
    # path is clear inside the node
    if node1 is node2:
        return clear_path_node(graph, reg, loc1, loc2)

    # If instructions are in different nodes, we also have to check the nodes
    # in the path between the two locations.
    # We try to return as early as possible to avoid unnecessary computations.
    if not clear_path_node(graph, reg, loc1, node1.ins_range[1]):
        return False
    path = build_path(graph, node1, node2)
    for node in path:
        locs = node.ins_range
        if not clear_path_node(graph, reg, locs[0], min(loc2, locs[1])):
            return False
    return True


def register_propagation(graph, du, ud):
    '''
    Propagate the temporary registers between instructions and remove them if
    necessary.
    We process the nodes of the graph in post order. For each instruction in
    the node, we look at the variables that it uses. For each of these
    variables we look where it is defined and if we can replace it with its
    definition.
    We have to be careful to the side effects some instructions may have.
    To do the propagation, we use the computed DU and UD chains.
    '''
    change = True
    while change:
        change = False
        for node in graph.get_rpo():
            for i, ins in node.get_loc_with_ins()[:]:
                # We make sure the ins has not been deleted since the start of
                # the iteration
                if ins not in node.get_ins():
                    continue
                for var in ins.get_used_vars():
                    # Get the list of locations this variable is defined at.
                    locs = ud.get((var, i), ())
                    # If the variable is uniquely defined for this instruction
                    # it may be eligible for propagation.
                    if len(locs) == 1:
                        loc = iter(locs).next()
                        # Methods parameters are defined with a location of -1.
                        if loc == -1:
                            continue
                        orig_ins = graph.get_ins_from_loc(loc)

                        # We only try to propagate constants and definition
                        # points which are used at only one location.
                        if len(du.get((var, loc), ())) > 1:
                            if not orig_ins.get_rhs().is_const():
                                continue

                        # We defined some instructions as not propagable.
                        # Actually this is the case only for array creation
                        # (new foo[x])
                        propag = orig_ins.is_propagable()
                        # We check that the propagation is safe for all the
                        # variables that are used in the instruction.
                        # The propagation is not safe if there is a side effect
                        # along the path from the definition of the variable
                        # to its use in the instruction, or if the variable may
                        # be redifined along this path.
                        for var2 in orig_ins.get_used_vars():
                            # loc is the location of the defined variable
                            # i is the location of the current instruction
                            if propag and not clear_path(graph, var2,
                                                         loc + 1, i):
                                propag = False
                        # We also check that the instruction itself is
                        # propagable. If the instruction has a side effect it
                        # cannot be propagated if there is another side effect
                        # along the path
                        if (propag and orig_ins.has_side_effect() and
                                    not clear_path(graph, None, loc + 1, i)):
                            propag = False
                        if propag:
                            ins.modify_rhs(var, orig_ins.get_rhs())
                            ud[(var, i)].remove(loc)
                            for var2 in orig_ins.get_used_vars():
                                # We update the UD chain of the variables we
                                # propagate. We also have to take the
                                # definition points of all the variables used
                                # by the instruction and update the DU chain
                                # with this information.
                                old_ud = ud.get((var2, loc))
                                # If the instruction use the same variable
                                # multiple times, the second+ time the ud chain
                                # will be None because already treated.
                                if old_ud is None:
                                    continue
                                ud.setdefault((var2, i), []).extend(old_ud)
                                ud.pop((var2, loc))

                                for def_loc in old_ud:
                                    du.get((var2, def_loc)).remove(loc)
                                    du.get((var2, def_loc)).append(i)

                            new_du = du.get((var, loc))
                            new_du.remove(i)
                            if len(new_du) == 0:
                                graph.remove_ins(loc)
                                change = True


def immediate_dominator(graph):
    '''
    Create a mapping of the nodes of a graph with their corresponding immediate
    dominator
    '''
    idom = dict([(n, None) for n in graph])
    for node in graph.get_rpo():
        for pred in graph.preds(node):
            if pred.num < node.num:
                idom[node] = common_dom(idom, idom[node], pred)
    return idom


def dominator_tree(graph, immediate_dominators):
    from dad.graph import Graph
    dom_tree = Graph()
    for n, idom_n in immediate_dominators.items():
        dom_tree.add_node(n)
        if idom_n:
            dom_tree.add_edge(idom_n, n)
    dom_tree.set_entry(graph.get_entry())
    dom_tree.set_exit(graph.get_exit())
    return dom_tree


class DummyNode(object):
    def __init__(self, name):
        self.name = name

    def get_loc_with_ins(self):
        return []

    def __repr__(self):
        return '%s-dumnode' % self.name

    def __str__(self):
        return '%s-dummynode' % self.name


def build_def_use(graph, lparams):
    '''
    Builds the Def-Use and Use-Def (DU/UD) chains of the variables of the
    method.
    '''
#    if 0:
#        dom_tree = dominator_tree(graph, immdoms)
#        dom_tree.draw(graph.entry.name, 'dad_graphs/dominators', False)

    # We insert two special nodes : entry & exit, to the graph.
    # This is done to simplify the reaching definition analysis.
    old_entry = graph.get_entry()
    old_exit = graph.get_exit()
    new_entry = DummyNode('entry')
    graph.add_node(new_entry)
    graph.add_edge(new_entry, old_entry)
    graph.set_entry(new_entry)
    if old_exit:
        new_exit = DummyNode('exit')
        graph.add_node(new_exit)
        graph.add_edge(old_exit, new_exit)
        graph.rpo.append(new_exit)

    analysis = BasicReachDef(graph, set(lparams))
    analysis.run()

    # The analysis is done, We can now remove the two special nodes.
    graph.remove_node(new_entry)
    if old_exit:
        graph.remove_node(new_exit)
    graph.set_entry(old_entry)

    UD = {}
    for node in graph.get_rpo():
        for i, ins in node.get_loc_with_ins():
            for var in ins.get_used_vars():
                # var not in analysis.def_to_loc: test that the register
                # exists. It is possible that it is not the case, when a
                # variable is of a type which is stored on multiple registers
                # e.g: a 'double' stored in v3 is also present in v4, so a call
                # to foo(v3), will in fact call foo(v3, v4).
                if var not in analysis.def_to_loc:
                    continue
                ldefs = analysis.defs[node]
                prior_def = -1
                for v in ldefs.get(var, set()):
                    if prior_def < v < i:
                        prior_def = v
                if prior_def >= 0:
                    UD.setdefault((var, i), []).append(prior_def)
                else:
                    intersect = analysis.def_to_loc[var].intersection(
                                                            analysis.R[node])
                    UD.setdefault((var, i), []).extend(intersect)
    DU = {}
    for var_loc, defs_loc in UD.items():
        var, loc = var_loc
        for def_loc in defs_loc:
            DU.setdefault((var, def_loc), []).append(loc)

#    dom_frontier = dominance_frontier(graph, immdoms)
#    phi_placement(graph, lparams, dom_frontier)
    return UD, DU



#!/usr/bin/env python

# This file is part of Androguard.
#
# Copyright (C) 2012, Anthony Desnos <desnos at t0t0.fr>
# All rights reserved.
#
# Androguard is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Androguard is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Androguard.  If not, see <http://www.gnu.org/licenses/>.

import sys, os

from optparse import OptionParser

from androguard.core import androconf
from androguard.core.bytecodes import apk, dvm
from androguard.core.analysis import analysis

sys.path.append("./elsim")
from elsim import elsim
from elsim.elsim_dalvik import ProxyDalvik, FILTERS_DALVIK_SIM
from elsim.elsim_dalvik import ProxyDalvikStringMultiple, ProxyDalvikStringOne, FILTERS_DALVIK_SIM_STRING

option_0 = { 'name' : ('-i', '--input'), 'help' : 'file : use these filenames', 'nargs' : 2 }
option_1 = { 'name' : ('-t', '--threshold'), 'help' : 'specify the threshold (0.0 to 1.0) to know if a method is similar. This option will impact on the filtering method. Because if you specify a higher value of the threshold, you will have more associations', 'nargs' : 1 }
option_2 = { 'name' : ('-c', '--compressor'), 'help' : 'specify the compressor (BZ2, ZLIB, SNAPPY, LZMA, XZ). The final result depends directly of the type of compressor. But if you use LZMA for example, the final result will be better, but it take more time', 'nargs' : 1 }
option_4 = { 'name' : ('-d', '--display'), 'help' : 'display all information about methods', 'action' : 'count' }
option_5 = { 'name' : ('-n', '--new'), 'help' : 'calculate the final score only by using the ratio of included methods', 'action' : 'count' }
option_6 = { 'name' : ('-e', '--exclude'), 'help' : 'exclude specific class name (python regexp)', 'nargs' : 1 }
option_7 = { 'name' : ('-s', '--size'), 'help' : 'exclude specific method below the specific size (specify the minimum size of a method to be used (it is the length (bytes) of the dalvik method)', 'nargs' : 1 }
option_8 = { 'name' : ('-x', '--xstrings'), 'help' : 'display similarities of strings', 'action' : 'count'  }
option_9 = { 'name' : ('-v', '--version'), 'help' : 'version of the API', 'action' : 'count' }
option_10 = { 'name' : ('-l', '--library'), 'help' : 'use python library (python) or specify the path of the shared library)', 'nargs' : 1 }

options = [option_0, option_1, option_2, option_4, option_5, option_6, option_7, option_8, option_9, option_10]

def check_one_file(a, d1, dx1, FS, threshold, file_input, view_strings=False, new=True, library=True) :
    d2 = None
    ret_type = androconf.is_android( file_input )
    if ret_type == "APK" :
        a = apk.APK( file_input )
        d2 = dvm.DalvikVMFormat( a.get_dex() )
    elif ret_type == "DEX" :
        d2 = dvm.DalvikVMFormat( open(file_input, "rb").read() )

    if d2 == None :
      return
    dx2 = analysis.VMAnalysis( d2 )

    el = elsim.Elsim( ProxyDalvik(d1, dx1), ProxyDalvik(d2, dx2), FS, threshold, options.compressor, libnative=library )
    el.show()
    print "\t--> methods: %f%% of similarities" % el.get_similarity_value(new)
    

    if options.display :
        print "SIMILAR methods:"
        diff_methods = el.get_similar_elements()
        for i in diff_methods :
            el.show_element( i )
            
        print "IDENTICAL methods:"
        new_methods = el.get_identical_elements()
        for i in new_methods :
            el.show_element( i )

        print "NEW methods:"
        new_methods = el.get_new_elements()
        for i in new_methods :
            el.show_element( i, False )

        print "DELETED methods:"
        del_methods = el.get_deleted_elements()
        for i in del_methods :
            el.show_element( i )
            
        print "SKIPPED methods:"
        skipped_methods = el.get_skipped_elements()
        for i in skipped_methods :
            el.show_element( i )
    
    if view_strings :
        els = elsim.Elsim( ProxyDalvikStringMultiple(d1, dx1),
                           ProxyDalvikStringMultiple(d2, dx2), 
                           FILTERS_DALVIK_SIM_STRING, 
                           threshold, 
                           options.compressor, 
                           libnative=library )
        #els = elsim.Elsim( ProxyDalvikStringOne(d1, dx1),
        #    ProxyDalvikStringOne(d2, dx2), FILTERS_DALVIK_SIM_STRING, threshold, options.compressor, libnative=library )
        els.show()
        print "\t--> strings: %f%% of similarities" % els.get_similarity_value(new)
    
        if options.display :
          print "SIMILAR strings:"
          diff_strings = els.get_similar_elements()
          for i in diff_strings :
            els.show_element( i )
            
          print "IDENTICAL strings:"
          new_strings = els.get_identical_elements()
          for i in new_strings :
            els.show_element( i )

          print "NEW strings:"
          new_strings = els.get_new_elements()
          for i in new_strings :
            els.show_element( i, False )

          print "DELETED strings:"
          del_strings = els.get_deleted_elements()
          for i in del_strings :
            els.show_element( i )
            
          print "SKIPPED strings:"
          skipped_strings = els.get_skipped_elements()
          for i in skipped_strings :
            els.show_element( i )
        

def check_one_directory(a, d1, dx1, FS, threshold, directory, view_strings=False, new=True, library=True) :
    for root, dirs, files in os.walk( directory, followlinks=True ) :
        if files != [] :
            for f in files :
                real_filename = root
                if real_filename[-1] != "/" :
                    real_filename += "/"
                real_filename += f

                print "filename: %s ..." % real_filename
                check_one_file(a, d1, dx1, FS, threshold, real_filename, view_strings, new, library)

############################################################
def main(options, arguments) :
    if options.input != None :
        a = None
        ret_type = androconf.is_android( options.input[0] )
        if ret_type == "APK" :
            a = apk.APK( options.input[0] )
            d1 = dvm.DalvikVMFormat( a.get_dex() )
        elif ret_type == "DEX" :
            d1 = dvm.DalvikVMFormat( open(options.input[0], "rb").read() )
        
        dx1 = analysis.VMAnalysis( d1 )
        
        threshold = None
        if options.threshold != None :
            threshold = float(options.threshold)

        FS = FILTERS_DALVIK_SIM
        FS[elsim.FILTER_SKIPPED_METH].set_regexp( options.exclude )
        FS[elsim.FILTER_SKIPPED_METH].set_size( options.size )
    
        new = True
        if options.new != None :
          new = False
        
        library = True
        if options.library != None :
            library = options.library
            if options.library == "python" :
                library = False

        if os.path.isdir( options.input[1] ) == False :
            check_one_file( a, d1, dx1, FS, threshold, options.input[1], options.xstrings, new, library )
        else :
            check_one_directory(a, d1, dx1, FS, threshold, options.input[1], options.xstrings, new, library )

    elif options.version != None :
        print "Androsim version %s" % androconf.ANDROGUARD_VERSION

if __name__ == "__main__" :
    parser = OptionParser()
    for option in options :
        param = option['name']
        del option['name']
        parser.add_option(*param, **option)

    options, arguments = parser.parse_args()
    sys.argv[:] = arguments
    main(options, arguments)
